@@ -1664,8 +1664,8 @@ pub trait Txn {
         fn cursor_set_last_value_get_value() -> value;
         /// Set cursor to key-value pair
         fn cursor_set_pair(key, value) -> bool;
-        /// Set cursor to key-value pair
-        fn cursor_set_key_search_value_get_value(key) -> value;
+        /// Set cursor to key and value or next greater value if not existent and get value
+        fn cursor_set_key_search_value_get_value(key, value) -> value;
         /// Move cursor to next value of current key and get value
         fn cursor_set_next_value_get_value() -> value;
         /// Move cursor to next value of current key and get value
@@ -2007,7 +2007,7 @@ macro_rules! cursor_method_impls {
         );
         $dupkey_macro!(
             cursor_set_key_search_value_get_value,
-            (key),
+            (key, value),
             value,
             MDB_cursor_op_MDB_GET_BOTH_RANGE
         );
