@@ -271,6 +271,127 @@ fn test_cursor_dupkey() {
     td.close().unwrap();
 }
 
+#[test]
+fn test_cursor_various() {
+    let db_opts = DbOptions::new()
+        .key_type::<str>()
+        .value_type::<str>()
+        .keys_duplicate()
+        .unnamed();
+    let td = tempdir().unwrap();
+    let env_builder = EnvBuilder::new().dir(td.path());
+    {
+        let mut env = unsafe { env_builder.open_rw() }.unwrap();
+        let db = unsafe { env.create_db(&db_opts).unwrap() };
+        let mut txn = env.txn_rw().unwrap();
+        txn.put(&db, "key1", "value11").unwrap();
+        txn.put(&db, "key1", "value13").unwrap();
+        txn.put(&db, "key1", "value15").unwrap();
+        txn.put(&db, "key3", "value31").unwrap();
+        txn.put(&db, "key3", "value33").unwrap();
+        txn.put(&db, "key3", "value35").unwrap();
+        txn.put(&db, "key5", "value51").unwrap();
+        txn.put(&db, "key5", "value53").unwrap();
+        txn.put(&db, "key5", "value55").unwrap();
+        txn.commit().unwrap();
+    }
+    {
+        let env = unsafe { env_builder.open_ro() }.unwrap();
+        let db = unsafe { env.open_db(&db_opts).unwrap() };
+        let txn = env.txn_ro().unwrap();
+        let cursor = txn.new_cursor(&db).unwrap();
+        assert_eq!(
+            txn.cursor_search_key_get_pair(&cursor, "key2").unwrap(),
+            Some(("key3", "value31"))
+        );
+        assert_eq!(
+            txn.cursor_set_next_value_get_value(&cursor).unwrap(),
+            Some("value33")
+        );
+        assert_eq!(txn.cursor_get_current_key(&cursor).unwrap(), Some("key3"));
+        assert_eq!(
+            txn.cursor_set_next_get_pair(&cursor).unwrap(),
+            Some(("key3", "value35"))
+        );
+        assert_eq!(
+            txn.cursor_get_current_value(&cursor).unwrap(),
+            Some("value35")
+        );
+        assert_eq!(txn.cursor_get_current_value_count(&cursor).unwrap(), 3);
+        assert_eq!(txn.cursor_set_next_value_get_value(&cursor).unwrap(), None);
+        assert_eq!(txn.cursor_set_first(&cursor).unwrap(), true);
+        assert_eq!(
+            txn.cursor_get_current_pair(&cursor).unwrap(),
+            Some(("key1", "value11"))
+        );
+        assert_eq!(
+            txn.cursor_set_first_get_pair(&cursor).unwrap(),
+            Some(("key1", "value11"))
+        );
+        assert_eq!(
+            txn.cursor_set_next_key_get_pair(&cursor).unwrap(),
+            Some(("key3", "value31"))
+        );
+        assert_eq!(txn.cursor_set_key(&cursor, "key4").unwrap(), false);
+        assert_eq!(txn.cursor_set_key(&cursor, "key5").unwrap(), true);
+        assert_eq!(txn.cursor_set_key_get_value(&cursor, "key4").unwrap(), None);
+        assert_eq!(
+            txn.cursor_set_key_get_value(&cursor, "key5").unwrap(),
+            Some("value51")
+        );
+        assert_eq!(
+            txn.cursor_set_prev_get_pair(&cursor).unwrap(),
+            Some(("key3", "value35"))
+        );
+        assert_eq!(
+            txn.cursor_set_prev_key_get_pair(&cursor).unwrap(),
+            Some(("key1", "value15"))
+        );
+        assert_eq!(
+            txn.cursor_set_prev_value_get_value(&cursor).unwrap(),
+            Some("value13")
+        );
+        assert_eq!(txn.cursor_set_prev_key_get_pair(&cursor).unwrap(), None);
+        assert_eq!(
+            txn.cursor_set_key_search_value_get_value(&cursor, "key3", "value34")
+                .unwrap(),
+            Some("value35")
+        );
+        assert_eq!(
+            txn.cursor_set_key_search_value_get_value(&cursor, "key3", "value36")
+                .unwrap(),
+            None
+        );
+        assert_eq!(txn.cursor_set_last(&cursor).unwrap(), true);
+        assert_eq!(
+            txn.cursor_get_current_value(&cursor).unwrap(),
+            Some("value55")
+        );
+        assert_eq!(txn.cursor_set_first_value(&cursor).unwrap(), true);
+        assert_eq!(
+            txn.cursor_set_last_get_pair(&cursor).unwrap(),
+            Some(("key5", "value55"))
+        );
+        assert_eq!(
+            txn.cursor_set_first_value_get_value(&cursor).unwrap(),
+            Some("value51")
+        );
+        assert_eq!(
+            txn.cursor_set_pair(&cursor, "key5", "value33").unwrap(),
+            false
+        );
+        assert_eq!(
+            txn.cursor_set_pair(&cursor, "key3", "value33").unwrap(),
+            true
+        );
+        assert_eq!(
+            txn.cursor_get_current_value(&cursor).unwrap(),
+            Some("value33")
+        );
+    }
+    td.close().unwrap();
+}
+
 #[test]
 fn test_cursor_delete() {
     let db_opts = DbOptions::new()
